#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -------------------------------------------------------------------------------------
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#   Gimp-Python - allows the writing of GIMP plug-ins in Python.
#   Copyright (C) 1997  James Henstridge <james@daa.com.au>

# MrQ Gimp-Batch.py

# Ver. 0.1 Initial relase Plugin based on the idea of Tin Tran Run Code on Visible Layers Script for GIMP (https://gimplearn.net/viewtopic.php?f=3&t=1027&start=10)
#                         and Saul Goode (Sg-Combine-Bg.scm - popular for animation, watermark or signature).
#                         Port for Gegl by Kevin Payne [kp24] & Claude Lion [cli345] http://gimpchat.com/viewtopic.php?f=9&t=18040&start=30#p264354 

# Example custom code:
# for script-fu: pdb.script_fu_fuzzy_border(image ,layer ,(255,255,255),16,1,4,0,100,0,1)
#                pdb.script_fu_old_photo(image, layer, 1, 20, 1,0,0)
# for GEGL:      gegl_batch(image, layer, "cubism");gegl_batch(image, layer, "alien-map") <=or=> gegl_batch(image, layer, "cubism alien-map")
#                !!! Recommended for reading: Learn GEGL graph (Gimp Tutorials)-> http://gimpchat.com/viewtopic.php?f=10&t=19463
# for Python     pdb.python_fu_foggify(image, layer, "", (240,180,70), 1,100);pdb.gimp_image_merge_visible_layers(image, 0)
#                comment: Plugin Fog finishes working with two layers you need to combine them - so you need to add a procedure pdb.gimp_image_merge_visible_layers

import sys, os
from gimpfu import *
import gettext

whoiam = os.path.abspath(sys.argv[0])
whoiamName = whoiam.split(os.sep)[-1][:-3]
locale_user_dir = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( whoiamName, locale_user_dir, unicode=True )

####################################################################################################################
################# START gegl_command by cli345 #####################################################################
####################################################################################################################

from sys import platform
from ctypes import *

def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')

def gegl_batch(image, drawable, gegl_graph_string):
	
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()

####################################################################################################
################# END gegl_command #################################################################
####################################################################################################

def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)
	
def batch_convert(image, layer,option,background,offx,offy,code, inputFolder, outputFolder,display):
    ''' Inverts the colors of the PNG and JPEG images of a folder.
    
    Parameters:
    img : image The current image (unused).
    layer : layer The layer of the image that is selected (unused).
    inputFolder : string The folder of the images that must be inverted.
    outputFolder : string The folder in which save the inverted images.
    '''
    # Iterate the folder
    for file in os.listdir(sys_file(inputFolder)):
        try:
            # Build the full file paths.
            inputPath = inputFolder + os.sep + file
            outputPath = outputFolder + os.sep + file
        
            # Open the file if is a JPEG or PNG image.
            image = None
            if(file.lower().endswith(('.png'))):
                image = pdb.file_png_load(inputPath, inputPath)
            if(file.lower().endswith(('.jpeg', '.jpg'))):
                image = pdb.file_jpeg_load(inputPath, inputPath)
            if(file.lower().endswith(('.bmp'))):
                image = pdb.pdb.file_bmp_load(inputPath, inputPath)              
            # Verify if the file is an image.
            if(image != None):
                # Invert the image.
                if(len(image.layers) > 0):
				
                    if option==0 or option==1:                    			
                        v=pdb.gimp_file_load_layer(image, background)					
                        pdb.gimp_image_add_layer (image, v, -1)
                        layer = image.layers[-1]
                        
                        if option==0: # COMBINE
                            pdb.gimp_layer_set_offsets(layer, offx,offy)
                            pdb.gimp_image_lower_layer_to_bottom(image, v)

                        if option==1: # OVERLAY
                            pdb.gimp_layer_set_offsets(v, offx,offy)
                        layer = pdb.gimp_image_merge_visible_layers(image, 0)
						
                    if option==2:

                        layer = image.layers[0]
                        #z=pdb.gimp_image_merge_visible_layers(image, 0)
                        exec(code)	
                    
                    # Save the image.
                    if(file.lower().endswith(('.png'))):
                        pdb.file_png_save(image, image.layers[-1], outputPath, outputPath, 0, 9, 0, 0, 0, 0, 0)
                    if(file.lower().endswith(('.jpeg', '.jpg'))):
                        pdb.file_jpeg_save(image, image.layers[-1], outputPath, outputPath, 0.9, 0, 0, 0, "Creating with GIMP", 0, 0, 0, 0)
                    if(file.lower().endswith(('.bmp'))):
                        gimp.pdb.file_bmp_save(image, image.layers[-1], outputPath, outputPath)						
								
        except Exception as err:
            gimp.message("Unexpected error: " + str(err))
	
    if display==1:
        gimp.Display(image)	
    if display==2:
        if option != 2:
            if option==0:
                x= "COMBINE"
            if option==1:
                x= "OVERLAY"				
            p ="Operation: " + x + "\n " + "Selected: "+str(background)+"\n "+"Input Folder: "+str(inputFolder)+"\n "+"Output Folder: "+str(outputFolder)+"\n "+"Offset in x: " +str(offx)+"\n "+"Offset in y: "+str(offy)
        if option==2:
            p ="Operation:Custom: " + "\n " + str(code)	
        info = pdb.gimp_text_fontname(image, None,  0.0, 0.0, " "+p+" ", 0, 1, 20, 1, "Arial")
        pdb.gimp_text_layer_set_color(info, (0,0,0))
        gimp_copy = info.copy(0)
        image.add_layer(gimp_copy, -1)
        pdb.gimp_edit_clear(gimp_copy)
        pdb.gimp_context_set_foreground((255,255,255))
        pdb.gimp_edit_fill(gimp_copy,2)
        pdb.gimp_image_raise_item_to_top(image, gimp_copy)	
        pdb.gimp_image_raise_item_to_top(image, info)
        ren = pdb.gimp_image_merge_down(image, info, 0)
        ren.name="Info"
        gimp.Display(image)
    pdb.gimp_image_resize_to_layers(image)
	
register(
    "python_fu_batch_conversion_using_gimp",
    "Batch conversion using Gimp Ver.0.1\nAccepts files type: bmp, jpg, png.",
    "Batch the operation of the bmp/jpeg/png images of a folder",
    "MareroQ",
    "GPL",
    "2021",
    "Batch convert...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "layer", "Input layer", None),
        (PF_OPTION, "option", "Operation: ", 0, [_("COMBINE\n (selected under the images)"),_("OVERLAY\n(selected over the images)"),_("CUSTOM")]),
        (PF_FILENAME, "background", _("Selected image (filename):"), os.getcwd()),
        (PF_SPINNER, "offx",  "Offset in x direction", 100, (0, 1000, 1)),
        (PF_SPINNER, "offy",  "Offset in y direction", 100, (0, 1000, 1)),
        (PF_STRING, "code", " # CUSTOM Code to Run:\n (separate pdb-semicolon';') ", "pdb.gimp_image_scale(image, 250, 250)"),		
        (PF_DIRNAME, "inputFolder", _("Input Folder"), os.getcwd() ),
        (PF_DIRNAME, "outputFolder", _("Output Folder"), os.getcwd() ),
        (PF_OPTION, "Display", "Display in Gimp: \n (the last one in \n the Input Folder) ", 0, [_("DON'T USE"),_("Display"),_("Display image & Info ")]),
    ],
    [],
    batch_convert,  menu = "<Image>/Tools/",domain = (whoiamName, locale_user_dir) )

main()
